@@ -1,10 +1,10 @@
-// Copyright © 2021 Rouven Spreckels <rs@qu1x.dev>
+// Copyright © 2021-2022 Rouven Spreckels <rs@qu1x.dev>
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
-use super::{Bits, SimdMask};
+use super::{Bits, Select, SimdMask};
 use core::{
 	fmt::{Debug, Octal},
 	hash::Hash,
@@ -13,7 +13,7 @@ use core::{
 	ops::{BitAnd, BitAndAssign, BitOr, BitOrAssign, BitXor, BitXorAssign, Not},
 	ops::{Index, IndexMut},
 	ops::{Shl, ShlAssign, Shr, ShrAssign},
-	simd::Select,
+	simd::{LaneCount, Simd, SupportedLaneCount},
 };
 
 mod u32;
@@ -24,9 +24,12 @@ mod u64;
 /// [`SimdReal`]: `super::SimdReal`
 pub trait SimdBits<B: Bits, const LANES: usize>
 where
+	LaneCount<LANES>: SupportedLaneCount,
 	Self: Clone + Copy + Default,
 	Self: PartialEq + Eq + PartialOrd + Ord,
-	Self: From<[B; LANES]> + AsRef<[B; LANES]> + AsMut<[B; LANES]>,
+	Self: From<Simd<B, LANES>> + Into<Simd<B, LANES>>,
+	Self: From<[B; LANES]> + Into<[B; LANES]>,
+	Self: AsRef<[B; LANES]> + AsMut<[B; LANES]>,
 	Self: Product<Self> + Sum<Self>,
 	for<'a> Self: Product<&'a Self> + Sum<&'a Self>,
 	Self: Hash,
