@@ -1,12 +1,17 @@
-// Copyright © 2021 Rouven Spreckels <rs@qu1x.dev>
+// Copyright © 2021-2022 Rouven Spreckels <rs@qu1x.dev>
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
-use super::SimdReal;
+use super::{ApproxEq, Select, SimdReal};
 use core::simd::{LaneCount, Mask, Simd, SupportedLaneCount, Swizzle, Swizzle2};
 
+#[cfg(feature = "libm")]
+use super::Real;
+#[cfg(not(feature = "libm"))]
+use std::simd::StdFloat;
+
 impl<const LANES: usize> SimdReal<f32, LANES> for Simd<f32, LANES>
 where
 	LaneCount<LANES>: SupportedLaneCount,
@@ -72,14 +77,14 @@ where
 	where
 		LaneCount<LANES>: SupportedLaneCount,
 	{
-		self.scatter(slice, idxs)
+		self.scatter(slice, idxs);
 	}
 	#[inline]
 	fn scatter_select(self, slice: &mut [f32], enable: Mask<isize, LANES>, idxs: Simd<usize, LANES>)
 	where
 		LaneCount<LANES>: SupportedLaneCount,
 	{
-		self.scatter_select(slice, enable, idxs)
+		self.scatter_select(slice, enable, idxs);
 	}
 
 	#[inline]
@@ -230,4 +235,95 @@ where
 	fn to_radians(self) -> Self {
 		self.to_radians()
 	}
+
+	#[cfg(feature = "libm")]
+	#[inline]
+	fn mul_add(self, a: Self, b: Self) -> Self {
+		self * a + b
+	}
+	#[cfg(not(feature = "libm"))]
+	#[inline]
+	fn mul_add(self, a: Self, b: Self) -> Self {
+		StdFloat::mul_add(self, a, b)
+	}
+	#[cfg(feature = "libm")]
+	fn sqrt(self) -> Self {
+		self.to_array().map(Real::sqrt).into()
+	}
+	#[cfg(not(feature = "libm"))]
+	#[inline]
+	fn sqrt(self) -> Self {
+		StdFloat::sqrt(self)
+	}
+	#[cfg(feature = "libm")]
+	#[inline]
+	fn floor(self) -> Self {
+		self.to_array().map(Real::floor).into()
+	}
+	#[cfg(not(feature = "libm"))]
+	#[inline]
+	fn floor(self) -> Self {
+		StdFloat::floor(self)
+	}
+	#[cfg(feature = "libm")]
+	#[inline]
+	fn ceil(self) -> Self {
+		self.to_array().map(Real::ceil).into()
+	}
+	#[cfg(not(feature = "libm"))]
+	#[inline]
+	fn ceil(self) -> Self {
+		StdFloat::ceil(self)
+	}
+	#[cfg(feature = "libm")]
+	#[inline]
+	fn round(self) -> Self {
+		self.to_array().map(Real::round).into()
+	}
+	#[cfg(not(feature = "libm"))]
+	#[inline]
+	fn round(self) -> Self {
+		StdFloat::round(self)
+	}
+	#[cfg(feature = "libm")]
+	#[inline]
+	fn trunc(self) -> Self {
+		self.to_array().map(Real::trunc).into()
+	}
+	#[cfg(not(feature = "libm"))]
+	#[inline]
+	fn trunc(self) -> Self {
+		StdFloat::trunc(self)
+	}
+	#[cfg(feature = "libm")]
+	#[inline]
+	fn fract(self) -> Self {
+		self.to_array().map(Real::fract).into()
+	}
+	#[cfg(not(feature = "libm"))]
+	#[inline]
+	fn fract(self) -> Self {
+		StdFloat::fract(self)
+	}
+}
+
+impl<const LANES: usize> Select<Mask<i32, LANES>> for Simd<f32, LANES>
+where
+	LaneCount<LANES>: SupportedLaneCount,
+{
+	#[inline]
+	fn select(mask: Mask<i32, LANES>, true_values: Self, false_values: Self) -> Self {
+		mask.select(true_values, false_values)
+	}
+}
+
+impl<const LANES: usize> ApproxEq<f32> for Simd<f32, LANES>
+where
+	LaneCount<LANES>: SupportedLaneCount,
+{
+	#[inline]
+	fn approx_eq(&self, other: &Self, epsilon: f32, ulp: u32) -> bool {
+		self.lanes_approx_eq(*other, Simd::splat(epsilon), Simd::splat(ulp))
+			.all()
+	}
 }
